<script setup lang="ts">
import { computed, ref } from 'vue';

import { useVbenDrawer, useVbenModal } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { addFullName, cloneDeep, getPopupContainer } from '@vben/utils';

import { InputSearch } from 'ant-design-vue';

import { useVbenForm } from '#/adapter/form';
import { addFixBorrow, getFixBorrow, updateFixBorrow } from '#/api/eims/fixture-borrow';
import { userProfile } from '#/api/system/profile';
import { getDeptTree, userList } from '#/api/system/user';
import fixModal from '#/views/eims/components/fixture-modal.vue';
import userModal from '#/views/eims/components/user-modal.vue';

import { drawerSchema } from './data';

const emit = defineEmits<{ reload: [] }>();

const isUpdate = ref(false);
const title = computed(() => {
  return isUpdate.value ? $t('pages.common.edit') : $t('pages.common.add');
});

const [BasicForm, formApi] = useVbenForm({
  commonConfig: {
    formItemClass: 'col-span-2',
    componentProps: {
      class: 'w-full'
    },
    labelWidth: 120
  },
  schema: drawerSchema(),
  showDefaultActions: false,
  wrapperClass: 'grid-cols-2'
});

const [BasicDrawer, drawerApi] = useVbenDrawer({
  onCancel: handleCancel,
  onConfirm: handleConfirm,
  async onOpenChange(isOpen) {
    if (!isOpen) {
      return null;
    }
    drawerApi.drawerLoading(true);
    const { id } = drawerApi.getData() as { id?: number | string };
    isUpdate.value = !!id;
    // 初始化
    await setupDeptSelect();

    // 从工具台账页面打开特殊处理
    const { fixturePage } = drawerApi.getData() as { fixturePage?: boolean };
    if (fixturePage) {
      formApi.updateSchema([
        {
          componentProps: {
            disabled: false
          },
          fieldName: 'fixtureName'
        }
      ]);
      await updateCustomData(drawerApi.getData());
    }

    formApi.updateSchema([
      {
        componentProps: {
          disabled: isUpdate.value || fixturePage
        },
        fieldName: 'fixtureName'
      }
    ]);
    // 更新 && 赋值
    if (isUpdate.value && id) {
      const record = await getFixBorrow(id);
      await formApi.setValues(record);
      if (isUpdate.value && record.borrowDept) {
        await setupUserOptions(record.borrowDept);
      }
    }

    drawerApi.drawerLoading(false);
  }
});

/**
 * 从设备台账页面打开更新部分数据
 * @param data
 */
async function updateCustomData(data: any) {
  const resp = await userProfile();
  const deptId = resp?.user?.deptId;
  await setupUserOptions(deptId);
  data.borrowDept = deptId;
  await formApi.setValues(data);
}

/**
 * 用户的加载
 */
async function setupUserOptions(deptId: any) {
  const params = { deptId };
  const userPageResult = await userList({
    pageNum: 1,
    pageSize: 500,
    ...params
  });
  const options = userPageResult.rows.map((item) => ({
    label: item.nickName || item.userName,
    value: item.userId
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '该部门下暂无用户';
  formApi.updateSchema([
    {
      componentProps: { options, placeholder, filterOption },
      fieldName: 'borrowUser'
    }
  ]);
}

/**
 * 初始化部门选择
 */
async function setupDeptSelect() {
  // updateSchema
  const deptTree = await getDeptTree();
  // 选中后显示在输入框的值 即父节点 / 子节点
  addFullName(deptTree, 'label', ' / ');
  formApi.updateSchema([
    {
      componentProps: (formModel) => ({
        class: 'w-full',
        fieldNames: {
          key: 'id',
          value: 'id',
          children: 'children'
        },
        getPopupContainer,
        async onSelect(deptId: number | string) {
          /** 根据部门ID加载用户 */
          await setupUserOptions(deptId);
          /** 变化后需要重新选择用户 */
          formModel.operatorId = undefined;
        },
        placeholder: '请选择',
        showSearch: true,
        treeData: deptTree,
        treeDefaultExpandAll: true,
        treeLine: { showLeafIcon: false },
        // 筛选的字段
        treeNodeFilterProp: 'label',
        // 选中后显示在输入框的值
        treeNodeLabelProp: 'fullName'
      }),
      fieldName: 'borrowDept'
    }
  ]);
}
async function handleConfirm() {
  try {
    drawerApi.drawerLoading(true);
    const { valid } = await formApi.validate();
    if (!valid) {
      return;
    }
    const data = cloneDeep(await formApi.getValues());
    await (isUpdate.value ? updateFixBorrow(data) : addFixBorrow(data));
    emit('reload');
    await handleCancel();
  } catch (error) {
    console.error(error);
  } finally {
    drawerApi.drawerLoading(false);
  }
}

async function handleCancel() {
  drawerApi.close();
  await formApi.resetForm();
}

// user modal
const [UserModal, userModalApi] = useVbenModal({
  connectedComponent: userModal,
  draggable: true,
  title: '选择成员'
});

function handleUserModal() {
  userModalApi.setData({});
  userModalApi.open();
}

/**
 * 更新选择的成员
 * @param user
 */
async function selectUser(user: any) {
  await formApi.setValues({ 'agentUser': user.userId, 'agentUserName': user.nickName });
}

// user modal
const [FixModal, fixModalApi] = useVbenModal({
  connectedComponent: fixModal,
  draggable: true,
  title: '选择工具'
});

function handleFixModal() {
  fixModalApi.setData({});
  fixModalApi.open();
}

/**
 * 更新选择的工具
 * @param fix
 */
async function selectFix(fix: any) {
  await formApi.setValues({ 'fixtureId': fix.id, 'fixtureName': fix.fixtureName });
}
</script>

<template>
  <BasicDrawer :close-on-click-modal="false" :title="title" class="w-[600px]">
    <BasicForm>
      <template #agentUserName="slotProps">
        <InputSearch :enter-button="true" placeholder="请选择" @search="handleUserModal" v-bind="slotProps" />
      </template>
      <template #fixtureName="slotProps">
        <InputSearch :enter-button="true" placeholder="请选择" @search="handleFixModal" v-bind="slotProps" />
      </template>
    </BasicForm>
    <UserModal class="w-[1200px]" @select-user="selectUser" />
    <FixModal class="w-[1200px]" @update-select="selectFix" />
  </BasicDrawer>
</template>
